@@ -263,28 +263,12 @@ static inline void populateActionSelection(QComboBox *list)
 	}
 }
 
-static inline void populateDeinterlaceModeSelection(QComboBox *list)
+template<class T>
+static inline void populateModeSelection(QComboBox *list,
+					 const std::map<T, std::string> &modes)
 {
 	list->clear();
-	for (const auto &[value, name] : deinterlaceModes) {
-		list->addItem(obs_module_text(name.c_str()),
-			      static_cast<int>(value));
-	}
-}
-
-static inline void populateDeinterlaceFieldOrderSelection(QComboBox *list)
-{
-	list->clear();
-	for (const auto &[value, name] : deinterlaceFieldOrders) {
-		list->addItem(obs_module_text(name.c_str()),
-			      static_cast<int>(value));
-	}
-}
-
-static inline void populateSettingsInputMethods(QComboBox *list)
-{
-	list->clear();
-	for (const auto &[value, name] : inputMethods) {
+	for (const auto &[value, name] : modes) {
 		list->addItem(obs_module_text(name.c_str()),
 			      static_cast<int>(value));
 	}
@@ -295,7 +279,7 @@ MacroActionSourceEdit::MacroActionSourceEdit(
 	: QWidget(parent),
 	  _sources(new SourceSelectionWidget(this, QStringList(), true)),
 	  _actions(new QComboBox()),
-	  _settingsButtons(new QComboBox()),
+	  _settingsButtons(new SourceSettingsButtonSelection(this)),
 	  _settingsLayout(new QHBoxLayout()),
 	  _settingsInputMethods(new QComboBox(this)),
 	  _manualSettingValue(new VariableTextEdit(this, 5, 1, 1)),
@@ -315,16 +299,18 @@ MacroActionSourceEdit::MacroActionSourceEdit(
 	auto sources = GetSourceNames();
 	sources.sort();
 	_sources->SetSourceNameList(sources);
-	populateDeinterlaceModeSelection(_deinterlaceMode);
-	populateDeinterlaceFieldOrderSelection(_deinterlaceOrder);
-	populateSettingsInputMethods(_settingsInputMethods);
+	populateModeSelection(_deinterlaceMode, deinterlaceModes);
+	populateModeSelection(_deinterlaceOrder, deinterlaceFieldOrders);
+	populateModeSelection(_settingsInputMethods, inputMethods);
 	_refreshSettingSelection->setToolTip(obs_module_text(
-		"AdvSceneSwitcher.action.source.refreshTooltip"));
+		"AdvSceneSwitcher.action.source.refresh.tooltip"));
 
 	QWidget::connect(_actions, SIGNAL(currentIndexChanged(int)), this,
 			 SLOT(ActionChanged(int)));
-	QWidget::connect(_settingsButtons, SIGNAL(currentIndexChanged(int)),
-			 this, SLOT(ButtonChanged(int)));
+	QWidget::connect(_settingsButtons,
+			 SIGNAL(SelectionChanged(const SourceSettingButton &)),
+			 this,
+			 SLOT(ButtonChanged(const SourceSettingButton &)));
 	QWidget::connect(_sources,
 			 SIGNAL(SourceChanged(const SourceSelection &)), this,
 			 SLOT(SourceChanged(const SourceSelection &)));
@@ -394,14 +380,11 @@ void MacroActionSourceEdit::UpdateEntryData()
 		return;
 	}
 
-	PopulateSourceButtonSelection(_settingsButtons,
-				      _entryData->_source.GetSource());
+	const auto weakSource = _entryData->_source.GetSource();
+	_settingsButtons->SetSelection(weakSource, _entryData->_button);
 	_actions->setCurrentIndex(static_cast<int>(_entryData->_action));
 	_sources->SetSource(_entryData->_source);
-	_sourceSettings->SetSource(_entryData->_source.GetSource());
-	_sourceSettings->SetSetting(_entryData->_setting);
-	_settingsButtons->setCurrentText(
-		QString::fromStdString(_entryData->_button.ToString()));
+	_sourceSettings->SetSelection(weakSource, _entryData->_setting);
 	_settingsString->setPlainText(_entryData->_settingsString);
 	_deinterlaceMode->setCurrentIndex(_deinterlaceMode->findData(
 		static_cast<int>(_entryData->_deinterlaceMode)));
@@ -425,9 +408,10 @@ void MacroActionSourceEdit::SourceChanged(const SourceSelection &source)
 		auto lock = LockContext();
 		_entryData->_source = source;
 	}
-	PopulateSourceButtonSelection(_settingsButtons,
-				      _entryData->_source.GetSource());
-	_sourceSettings->SetSource(_entryData->_source.GetSource());
+
+	const auto weakSource = _entryData->_source.GetSource();
+	_sourceSettings->SetSource(weakSource);
+	_settingsButtons->SetSource(weakSource);
 	SetWidgetVisibility();
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -435,24 +419,15 @@ void MacroActionSourceEdit::SourceChanged(const SourceSelection &source)
 
 void MacroActionSourceEdit::ActionChanged(int value)
 {
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	auto lock = LockContext();
+	GUARD_LOADING_AND_LOCK();
 	_entryData->_action = static_cast<MacroActionSource::Action>(value);
 	SetWidgetVisibility();
 }
 
-void MacroActionSourceEdit::ButtonChanged(int idx)
+void MacroActionSourceEdit::ButtonChanged(const SourceSettingButton &button)
 {
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	auto lock = LockContext();
-	_entryData->_button = qvariant_cast<SourceSettingButton>(
-		_settingsButtons->itemData(idx));
+	GUARD_LOADING_AND_LOCK();
+	_entryData->_button = button;
 }
 
 void MacroActionSourceEdit::GetSettingsClicked()
@@ -479,11 +454,7 @@ void MacroActionSourceEdit::GetSettingsClicked()
 
 void MacroActionSourceEdit::SettingsStringChanged()
 {
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	auto lock = LockContext();
+	GUARD_LOADING_AND_LOCK();
 	_entryData->_settingsString =
 		_settingsString->toPlainText().toStdString();
 
@@ -493,22 +464,14 @@ void MacroActionSourceEdit::SettingsStringChanged()
 
 void MacroActionSourceEdit::DeinterlaceModeChanged(int idx)
 {
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	auto lock = LockContext();
+	GUARD_LOADING_AND_LOCK();
 	_entryData->_deinterlaceMode = static_cast<obs_deinterlace_mode>(
 		_deinterlaceMode->itemData(idx).toInt());
 }
 
 void MacroActionSourceEdit::DeinterlaceOrderChanged(int idx)
 {
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	auto lock = LockContext();
+	GUARD_LOADING_AND_LOCK();
 	_entryData->_deinterlaceOrder =
 		static_cast<obs_deinterlace_field_order>(
 			_deinterlaceOrder->itemData(idx).toInt());
@@ -516,21 +479,13 @@ void MacroActionSourceEdit::DeinterlaceOrderChanged(int idx)
 
 void MacroActionSourceEdit::SelectionChanged(const TempVariableRef &var)
 {
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	auto lock = LockContext();
+	GUARD_LOADING_AND_LOCK();
 	_entryData->_tempVar = var;
 }
 
 void MacroActionSourceEdit::SettingsInputMethodChanged(int idx)
 {
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	auto lock = LockContext();
+	GUARD_LOADING_AND_LOCK();
 	_entryData->_settingsInputMethod =
 		static_cast<MacroActionSource::SettingsInputMethod>(
 			_settingsInputMethods->itemData(idx).toInt());
@@ -539,21 +494,13 @@ void MacroActionSourceEdit::SettingsInputMethodChanged(int idx)
 
 void MacroActionSourceEdit::SelectionChanged(const SourceSetting &setting)
 {
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	auto lock = LockContext();
+	GUARD_LOADING_AND_LOCK();
 	_entryData->_setting = setting;
 }
 
 void MacroActionSourceEdit::ManualSettingsValueChanged()
 {
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	auto lock = LockContext();
+	GUARD_LOADING_AND_LOCK();
 	_entryData->_manualSettingValue =
 		_manualSettingValue->toPlainText().toStdString();
 
